@@ -101,15 +101,17 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   int red_band=0, green_band=0, blue_band=0, alpha_band=0;
   int band_count, band_numbers[4];
   GDALDatasetH hDS = hDSVoid;
-  GDALColorTableH hColorMap;
+  GDALColorTableH hColorMap=NULL;
   GDALRasterBandH hBand1=NULL, hBand2=NULL, hBand3=NULL, hBandAlpha=NULL;
   int bHaveRGBNoData = FALSE;
   int nNoData1=-1,nNoData2=-1,nNoData3=-1;
   rasterBufferObj *mask_rb = NULL;
+  rasterBufferObj s_mask_rb;
   if(layer->mask) {
     int ret;
     layerObj *maskLayer = GET_LAYER(map, msGetLayerIndex(map,layer->mask));
-    mask_rb = msSmallCalloc(1,sizeof(rasterBufferObj)); 
+    mask_rb = &s_mask_rb;
+    memset(mask_rb, 0, sizeof(s_mask_rb));
     ret = MS_IMAGE_RENDERER(maskLayer->maskimage)->getRasterBufferHandle(maskLayer->maskimage,mask_rb);
     if(ret != MS_SUCCESS)
       return -1;
@@ -331,24 +333,32 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
 
   /*
    * Set up the band selection.  We look for a BANDS directive in the
-   * the PROCESSING options.  If not found we default to red=1 or
-   * red=1,green=2,blue=3 or red=1,green=2,blue=3,alpha=4.
+   * the PROCESSING options.  If not found we default to grey=1, grey=1,alpha=2,
+   * red=1,green=2,blue=3 or red=1,green=2,blue=3,alpha>=4.
    */
 
   if( CSLFetchNameValue( layer->processing, "BANDS" ) == NULL ) {
+    const int gdal_band_count = GDALGetRasterCount( hDS );
     red_band = 1;
 
-    if( GDALGetRasterCount( hDS ) >= 4
-        && GDALGetRasterColorInterpretation(
-          GDALGetRasterBand( hDS, 4 ) ) == GCI_AlphaBand )
-      alpha_band = 4;
+    if( gdal_band_count >= 4 )
+    {
+        /* The alpha band is not necessarily the 4th one */
+        for( i = 4; i <= gdal_band_count; i ++ ) {
+            if( GDALGetRasterColorInterpretation(
+                GDALGetRasterBand( hDS, i ) ) == GCI_AlphaBand ) {
+                alpha_band = i;
+                break;
+            }
+        }
+    }
 
-    if( GDALGetRasterCount( hDS ) >= 3 ) {
+    if( gdal_band_count >= 3 ) {
       green_band = 2;
       blue_band = 3;
     }
 
-    if( GDALGetRasterCount( hDS ) == 2
+    if( gdal_band_count == 2
         && GDALGetRasterColorInterpretation(
           GDALGetRasterBand( hDS, 2 ) ) == GCI_AlphaBand )
       alpha_band = 2;
@@ -530,6 +540,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
       msSetError(MS_MISCERR,
                  "Unknown RANGE_COLORSPACE \"%s\", expecting RGB or HSL",
                  "drawGDAL()", pszRangeColorspace);
+      GDALDestroyColorTable( hColorMap );
       return -1;
     }
 
@@ -624,6 +635,9 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   if( pabyRaw1 == NULL ) {
     msSetError(MS_MEMERR, "Allocating work image of size %dx%dx%d failed.",
                "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize, band_count );
+
+    if( hColorMap != NULL )     
+      GDALDestroyColorTable( hColorMap );
     return -1;
   }
 
@@ -648,6 +662,8 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
                       &bHaveRGBNoData,
                       &nNoData1, &nNoData2, &nNoData3 ) == -1 ) {
     free( pabyRaw1 );
+    if( hColorMap != NULL )     
+      GDALDestroyColorTable( hColorMap );
     return -1;
   }
 
@@ -666,6 +682,8 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
         (nMaskFlags & GMF_PER_DATASET) != 0 &&
         (nMaskFlags & (GMF_NODATA|GMF_ALL_VALID)) == 0 ) {
       CPLErr eErr;
+      unsigned char * pabyOrig = pabyRaw1;
+
 
       if( layer->debug )
         msDebug( "msDrawGDAL(): using GDAL mask band for alpha.\n" );
@@ -673,13 +691,16 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
       band_count++;
 
       pabyRaw1 = (unsigned char *)
-                 realloc(pabyRaw1,dst_xsize * dst_ysize * band_count);
+                 realloc(pabyOrig,dst_xsize * dst_ysize * band_count);
 
       if( pabyRaw1 == NULL ) {
         msSetError(MS_MEMERR,
                    "Allocating work image of size %dx%dx%d failed.",
                    "msDrawRasterLayerGDAL()",
                    dst_xsize, dst_ysize, band_count );
+        free(pabyOrig);
+        if( hColorMap != NULL )     
+          GDALDestroyColorTable( hColorMap );
         return -1;
       }
 
@@ -701,6 +722,8 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
         msSetError( MS_IOERR, "GDALRasterIO() failed: %s",
                     "drawGDAL()", CPLGetLastErrorMsg() );
         free( pabyRaw1 );
+        if( hColorMap != NULL )     
+          GDALDestroyColorTable( hColorMap );
         return -1;
       }
 
@@ -831,8 +854,6 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   /*
   ** Cleanup
   */
-
-  msFree( mask_rb );
   free( pabyRaw1 );
 
   if( hColorMap != NULL )
@@ -845,7 +866,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
 /*                          ParseDefaultLUT()                           */
 /************************************************************************/
 
-static int ParseDefaultLUT( const char *lut_def, GByte *lut )
+static int ParseDefaultLUT( const char *lut_def, GByte *lut, int nMaxValIn )
 
 {
   const char *lut_read;
@@ -862,11 +883,11 @@ static int ParseDefaultLUT( const char *lut_def, GByte *lut )
     while( isspace(*lut_read) )
       lut_read++;
 
-    /* if we are at end, assum 255:255 */
+    /* if we are at end, assume nMaxValIn:255 */
     if( *lut_read == '\0' ) {
       all_done = TRUE;
-      if ( last_in != 255 ) {
-        this_in = 255;
+      if ( last_in != nMaxValIn ) {
+        this_in = nMaxValIn;
         this_out = 255;
       }
     }
@@ -889,7 +910,7 @@ static int ParseDefaultLUT( const char *lut_def, GByte *lut )
         lut_read++;
     }
 
-    this_in = MS_MAX(0,MS_MIN(255,this_in));
+    this_in = MS_MAX(0,MS_MIN(nMaxValIn,this_in));
     this_out = MS_MAX(0,MS_MIN(255,this_out));
 
     /* apply linear values from last in:out to this in:out */
@@ -955,7 +976,7 @@ static int LutFromGimpLine( char *lut_line, GByte *lut )
 
   CSLDestroy( tokens );
 
-  return ParseDefaultLUT( wrkLUTDef, lut );
+  return ParseDefaultLUT( wrkLUTDef, lut, 255 );
 }
 
 /************************************************************************/
@@ -978,6 +999,7 @@ static int ParseGimpLUT( const char *lut_def, GByte *lut, int iColorIndex )
     msSetError(MS_MISCERR,
                "GIMP curve file appears corrupt.",
                "ParseGimpLUT()" );
+    CSLDestroy( lines );
     return -1;
   }
 
@@ -1005,19 +1027,18 @@ static int ParseGimpLUT( const char *lut_def, GByte *lut, int iColorIndex )
 }
 
 /************************************************************************/
-/*                              ApplyLUT()                              */
+/*                              LoadLUT()                               */
 /*                                                                      */
-/*      Apply a LUT according to RFC 21.                                */
+/*      Load a LUT according to RFC 21.                                 */
 /************************************************************************/
 
-static int ApplyLUT( int iColorIndex, layerObj *layer,
-                     GByte *buffer, int buf_xsize, int buf_ysize )
+static int LoadLUT( layerObj *layer, int iColorIndex, char** ppszLutDef )
 
 {
   const char *lut_def;
   char key[20], lut_def_fromfile[2500];
-  GByte lut[256];
-  int   err, i;
+
+  *ppszLutDef = NULL;
 
   /* -------------------------------------------------------------------- */
   /*      Get lut specifier from processing directives.  Do nothing if    */
@@ -1062,25 +1083,159 @@ static int ApplyLUT( int iColorIndex, layerObj *layer,
     lut_def = lut_def_fromfile;
   }
 
+  *ppszLutDef = msStrdup(lut_def);
+
+  return 0;
+
+}
+
+/************************************************************************/
+/*                              FreeLUTs()                              */
+/************************************************************************/
+
+static void FreeLUTs(char** apszLUTs)
+{
+    int i;
+    for( i = 0; i < 4; i++ )
+        msFree(apszLUTs[i]);
+    msFree(apszLUTs);
+}
+
+/************************************************************************/
+/*                            LoadLUTs()                                */
+/*                                                                      */
+/* Return an array of 4 strings (some possibly NULL) with loaded LUTs   */
+/* or NULL in case of failure.                                          */
+/************************************************************************/
+
+static char** LoadLUTs(layerObj *layer, int band_count)
+{
+    int i;
+    char** apszLUTs;
+    
+    assert(band_count <= 4);
+
+    apszLUTs = (char**) msSmallCalloc( 4, sizeof(char*) );
+    for( i = 0; i < band_count; i++ )
+    {
+        if( LoadLUT( layer, i+1, &apszLUTs[i] ) != 0 )
+        {
+            FreeLUTs(apszLUTs);
+            return NULL;
+        }
+    }
+
+    return apszLUTs;
+}
+
+/************************************************************************/
+/*                  GetDataTypeAppropriateForLUTS()                     */
+/*                                                                      */
+/*      This does a quick examination of the LUT strings to determine   */
+/*      if they have input values > 255, in which case the raster data  */
+/*      must be queries on 16-bits.                                     */
+/************************************************************************/
+
+static GDALDataType GetDataTypeAppropriateForLUTS(char** apszLUTs)
+{
+    GDALDataType eDT = GDT_Byte;
+    int i;
+    for( i = 0; i < 4; i++ )
+    {
+        const char* pszLastTuple;
+        int nLastInValue;
+        if( apszLUTs[i] == NULL )
+            continue;
+        if( EQUALN(apszLUTs[i],"# GIMP",6) )
+            continue;
+        /* Find last in:out tuple in string */
+        pszLastTuple = strrchr( apszLUTs[i], ',' );
+        if( pszLastTuple == NULL )
+            pszLastTuple = apszLUTs[i];
+        else
+            pszLastTuple ++;
+        while( *pszLastTuple == ' ' )
+            pszLastTuple ++;
+        nLastInValue = atoi(pszLastTuple);
+        if( nLastInValue > 255 )
+        {
+            eDT = GDT_UInt16;
+            break;
+        }
+    }
+    return eDT;
+}
+
+/************************************************************************/
+/*                              ApplyLUT()                              */
+/************************************************************************/
+
+static int ApplyLUT( int iColorIndex, const char* lut_def,
+                     const void* pInBuffer, GDALDataType eDT,
+                     GByte* pabyOutBuffer,
+                     int nPixelCount )
+{
+  int i, err;
+  GByte byteLut[256];
+  GByte* uint16Lut = NULL;
+
+  assert( eDT == GDT_Byte || eDT == GDT_UInt16 );
+  
+  if( lut_def == NULL )
+  {
+      if( pInBuffer != pabyOutBuffer )
+      {
+        GDALCopyWords( (void*)pInBuffer, eDT, GDALGetDataTypeSize(eDT) / 8,
+                        pabyOutBuffer, GDT_Byte, 1,
+                        nPixelCount );
+      }
+      return 0;
+  }
+
   /* -------------------------------------------------------------------- */
   /*      Parse the LUT description.                                      */
   /* -------------------------------------------------------------------- */
   if( EQUALN(lut_def,"# GIMP",6) ) {
-    err = ParseGimpLUT( lut_def, lut, iColorIndex );
+    if( eDT != GDT_Byte ) {
+      msSetError(MS_MISCERR,
+                 "Cannot apply a GIMP LUT on a 16-bit buffer",
+                 "ApplyLUT()");
+      return -1;
+    }
+    err = ParseGimpLUT( lut_def, byteLut, iColorIndex );
   } else {
-    err = ParseDefaultLUT( lut_def, lut );
+    if( eDT == GDT_Byte )
+        err = ParseDefaultLUT( lut_def, byteLut, 255 );
+    else
+    {
+        uint16Lut = (GByte*) malloc( 65536 );
+        if( uint16Lut == NULL )
+        {
+            msSetError(MS_MEMERR,
+                 "Cannot allocate 16-bit LUT",
+                 "ApplyLUT()" );
+            return -1;
+        }
+        err = ParseDefaultLUT( lut_def, uint16Lut, 65535 );
+    }
   }
 
-  if( err != 0 )
-    return err;
-
   /* -------------------------------------------------------------------- */
   /*      Apply LUT.                                                      */
   /* -------------------------------------------------------------------- */
-  for( i = buf_xsize * buf_ysize - 1; i >= 0; i-- )
-    buffer[i] = lut[buffer[i]];
-
-  return 0;
+  if( eDT == GDT_Byte )
+  {
+    for( i = 0; i < nPixelCount; i++ )
+        pabyOutBuffer[i] = byteLut[((GByte*)pInBuffer)[i]];
+  }
+  else
+  {
+      for( i = 0; i < nPixelCount; i++ )
+        pabyOutBuffer[i] = uint16Lut[((GUInt16*)pInBuffer)[i]];
+      free(uint16Lut);
+  }
+  
+  return err;
 }
 
 /************************************************************************/
@@ -1104,6 +1259,7 @@ LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
   int    iColorIndex, result_code=0;
   CPLErr eErr;
   float *pafWholeRawData;
+  char** papszLUTs;
 
   /* -------------------------------------------------------------------- */
   /*      If we have no alpha band, but we do have three input            */
@@ -1138,10 +1294,38 @@ LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
       && CSLFetchNameValue( layer->processing, "SCALE_2" ) == NULL
       && CSLFetchNameValue( layer->processing, "SCALE_3" ) == NULL
       && CSLFetchNameValue( layer->processing, "SCALE_4" ) == NULL ) {
+      
+    GDALDataType eDT;
+    GUInt16* panBuffer = NULL;
+    void* pBuffer;
+
+    papszLUTs = LoadLUTs(layer, band_count);
+    if( papszLUTs == NULL ) {
+        return -1;
+    }
+
+    eDT = GetDataTypeAppropriateForLUTS(papszLUTs);
+    if( eDT == GDT_UInt16 ) {
+        panBuffer =
+            (GUInt16 *) malloc(sizeof(GUInt16) * dst_xsize * dst_ysize * band_count );
+
+        if( panBuffer == NULL ) {
+            msSetError(MS_MEMERR,
+                    "Allocating work uint16 image of size %dx%dx%d failed.",
+                    "msDrawRasterLayerGDAL()",
+                    dst_xsize, dst_ysize, band_count );
+            FreeLUTs(papszLUTs);
+            return -1;
+        }
+        pBuffer = panBuffer;
+    }
+    else
+        pBuffer = pabyWholeBuffer;
+
     eErr = GDALDatasetRasterIO( hDS, GF_Read,
                                 src_xoff, src_yoff, src_xsize, src_ysize,
-                                pabyWholeBuffer,
-                                dst_xsize, dst_ysize, GDT_Byte,
+                                pBuffer,
+                                dst_xsize, dst_ysize, eDT,
                                 band_count, band_numbers,
                                 0,0,0);
 
@@ -1150,17 +1334,24 @@ LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
                   "GDALDatasetRasterIO() failed: %s",
                   "drawGDAL()",
                   CPLGetLastErrorMsg() );
+      FreeLUTs(papszLUTs);
+      msFree(panBuffer);
       return -1;
     }
 
     for( iColorIndex = 0;
          iColorIndex < band_count && result_code == 0; iColorIndex++ ) {
-      result_code = ApplyLUT( iColorIndex+1, layer,
-                              pabyWholeBuffer
-                              + dst_xsize*dst_ysize*iColorIndex,
-                              dst_xsize, dst_ysize );
+      result_code = ApplyLUT( iColorIndex+1,
+                              papszLUTs[iColorIndex],
+                              (GByte*)pBuffer
+                                + dst_xsize*dst_ysize*iColorIndex*(GDALGetDataTypeSize(eDT)/8),
+                              eDT,
+                              pabyWholeBuffer + dst_xsize*dst_ysize*iColorIndex,
+                              dst_xsize * dst_ysize );
     }
 
+    FreeLUTs(papszLUTs);
+    msFree(panBuffer);
     return result_code;
   }
 
@@ -1209,6 +1400,18 @@ LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
     return -1;
   }
 
+  papszLUTs = LoadLUTs(layer, band_count);
+  if( papszLUTs == NULL ) {
+    free( pafWholeRawData );
+    return -1;
+  }
+
+  if( GetDataTypeAppropriateForLUTS(papszLUTs) != GDT_Byte ) {
+    msDebug( "LoadGDALImage(%s): One of the LUT contains a input value > 255.\n"
+             "This is not properly supported in combination with SCALE\n",
+             layer->name );
+  }
+
   /* -------------------------------------------------------------------- */
   /*      Fetch the scale processing option.                              */
   /* -------------------------------------------------------------------- */
@@ -1238,6 +1441,8 @@ LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
         dfScaleMin = dfScaleMax = 0.0;
       } else if( CSLCount(papszTokens) != 2 ) {
         free( pafWholeRawData );
+        CSLDestroy( papszTokens );
+        FreeLUTs( papszLUTs );
         msSetError( MS_MISCERR,
                     "SCALE PROCESSING option unparsable for layer %s.",
                     "msDrawGDAL()",
@@ -1322,15 +1527,21 @@ LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
     /* -------------------------------------------------------------------- */
     /*      Apply LUT if there is one.                                      */
     /* -------------------------------------------------------------------- */
-    result_code = ApplyLUT( iColorIndex+1, layer,
-                            pabyBuffer, dst_xsize, dst_ysize );;
+    result_code = ApplyLUT( iColorIndex+1,
+                            papszLUTs[iColorIndex],
+                            pabyBuffer,
+                            GDT_Byte,
+                            pabyBuffer,
+                            dst_xsize * dst_ysize );
     if( result_code == -1 ) {
       free( pafWholeRawData );
+      FreeLUTs( papszLUTs );
       return result_code;
     }
   }
 
   free( pafWholeRawData );
+  FreeLUTs( papszLUTs );
 
   return result_code;
 }
@@ -1509,10 +1720,12 @@ msDrawRasterLayerGDAL_RawMode(
   GInt16 *i_nodatas = NULL;
   int got_nodata=FALSE;
   rasterBufferObj *mask_rb = NULL;
+  rasterBufferObj s_mask_rb;
   if(layer->mask) {
     int ret;
     layerObj *maskLayer = GET_LAYER(map, msGetLayerIndex(map,layer->mask));
-    mask_rb = msSmallCalloc(1,sizeof(rasterBufferObj)); 
+    mask_rb = &s_mask_rb;
+    memset(mask_rb, 0, sizeof(s_mask_rb));
     ret = MS_IMAGE_RENDERER(maskLayer->maskimage)->getRasterBufferHandle(maskLayer->maskimage,mask_rb);
     if(ret != MS_SUCCESS)
       return -1;
@@ -1585,7 +1798,7 @@ msDrawRasterLayerGDAL_RawMode(
   }
 
   if( !got_nodata ) {
-    msFree( f_nodatas );
+    free( f_nodatas );
     f_nodatas = NULL;
   } else if( eDataType == GDT_Byte ) {
     b_nodatas = (unsigned char *) f_nodatas;
@@ -1606,6 +1819,8 @@ msDrawRasterLayerGDAL_RawMode(
     msSetError(MS_MEMERR,
                "Allocating work image of size %dx%d failed.",
                "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize );
+    free( band_list );
+    free( f_nodatas );
     return -1;
   }
 
@@ -1679,7 +1894,6 @@ msDrawRasterLayerGDAL_RawMode(
     }
   }
 
-  msFree( mask_rb );
   free( pBuffer );
   free( f_nodatas );
 
@@ -1717,13 +1931,15 @@ msDrawRasterLayerGDAL_16BitClassification(
   unsigned char *rb_cmap[4];
   CPLErr eErr;
   rasterBufferObj *mask_rb = NULL;
+  rasterBufferObj s_mask_rb;
   int lastC;
   struct mstimeval starttime, endtime;
 
   if(layer->mask) {
     int ret;
     layerObj *maskLayer = GET_LAYER(map, msGetLayerIndex(map,layer->mask));
-    mask_rb = msSmallCalloc(1,sizeof(rasterBufferObj)); 
+    mask_rb = &s_mask_rb;
+    memset(mask_rb, 0, sizeof(s_mask_rb));
     ret = MS_IMAGE_RENDERER(maskLayer->maskimage)->getRasterBufferHandle(maskLayer->maskimage,mask_rb);
     if(ret != MS_SUCCESS)
       return -1;
@@ -1798,6 +2014,7 @@ msDrawRasterLayerGDAL_16BitClassification(
       dfScaleMin = dfScaleMax = 0.0;
     } else if( CSLCount(papszTokens) != 2 ) {
       free( pafRawData );
+      CSLDestroy( papszTokens );
       msSetError( MS_MISCERR,
                   "SCALE PROCESSING option unparsable for layer %s.",
                   "msDrawGDAL()",
@@ -1975,7 +2192,6 @@ msDrawRasterLayerGDAL_16BitClassification(
   free( rb_cmap[1] );
   free( rb_cmap[2] );
   free( rb_cmap[3] );
-  msFree( mask_rb );
 
   assert( k == dst_xsize * dst_ysize );
 
@@ -2052,6 +2268,8 @@ int *msGetGDALBandList( layerObj *layer, void *hDS,
       *band_count = file_bands;
 
     band_list = (int *) malloc(sizeof(int) * *band_count );
+
+    /* FIXME MS_CHECK_ALLOC leaks papszItems */
     MS_CHECK_ALLOC(band_list, sizeof(int) * *band_count, NULL);
 
     for( i = 0; i < *band_count; i++ )
