@@ -31,6 +31,10 @@
 
 #ifdef USE_GEOS
 
+// To avoid accidental use of non reentrant GEOS API.
+// (check only effective in GEOS >= 3.5)
+#define GEOS_USE_ONLY_R_API
+
 #include <geos_c.h>
 
 /*
@@ -64,7 +68,7 @@ static inline GEOSContextHandle_t msGetGeosContextHandle()
 #include "mapthread.h"
 typedef struct geos_thread_info {
   struct geos_thread_info *next;
-  int             thread_id;
+  void*             thread_id;
   GEOSContextHandle_t        geos_handle;
 } geos_thread_info_t;
 
@@ -74,7 +78,7 @@ static GEOSContextHandle_t msGetGeosContextHandle()
 {
   geos_thread_info_t *link;
   GEOSContextHandle_t ret_obj;
-  int        thread_id;
+  void*        thread_id;
 
   msAcquireLock( TLOCK_GEOS );
 
@@ -269,14 +273,17 @@ static GEOSGeom msGEOSShape2Geometry_simplepolygon(shapeObj *shape, int r, int *
   if(numInnerRings > 0) {
     k = 0; /* inner ring counter */
 
-    innerRings = malloc(numInnerRings*sizeof(GEOSGeom));
-    if(!innerRings) return NULL; /* todo, this will leak memory (outerRing) */
+    innerRings = msSmallMalloc(numInnerRings*sizeof(GEOSGeom));
 
     for(j=0; j<shape->numlines; j++) {
       if(innerList[j] == MS_FALSE) continue;
 
       coords = GEOSCoordSeq_create_r(handle,shape->line[j].numpoints, 2); /* todo handle z's */
-      if(!coords) return NULL; /* todo, this will leak memory (shell + allocated holes) */
+      if(!coords) {
+        free(innerRings);
+        free(innerList);
+        return NULL; /* todo, this will leak memory (shell + allocated holes) */
+      }
 
       for(i=0; i<shape->line[j].numpoints; i++) {
         GEOSCoordSeq_setX_r(handle,coords, i, shape->line[j].point[i].x);
@@ -316,8 +323,7 @@ static GEOSGeom msGEOSShape2Geometry_polygon(shapeObj *shape)
   if(numOuterRings == 1) {
     g = msGEOSShape2Geometry_simplepolygon(shape, lastOuterRing, outerList);
   } else { /* a true multipolygon */
-    polygons = malloc(numOuterRings*sizeof(GEOSGeom));
-    if(!polygons) return NULL;
+    polygons = msSmallMalloc(numOuterRings*sizeof(GEOSGeom));
 
     j = 0; /* part counter */
     for(i=0; i<shape->numlines; i++) {
@@ -714,6 +720,7 @@ void msGEOSFreeGeometry(shapeObj *shape)
 
   g = (GEOSGeom) shape->geometry;
   GEOSGeom_destroy_r(handle,g);
+  shape->geometry = NULL;
 #else
   msSetError(MS_GEOSERR, "GEOS support is not available.", "msGEOSFreeGEOSGeom()");
   return;
