@@ -290,7 +290,10 @@ int mvtWriteShape( layerObj *layer, shapeObj *shape, VectorTile__Tile__Layer *mv
       if( item->type && EQUAL(item->type,"Integer")) {
         mvt_value->int_value = atoi(value->value);
         mvt_value->has_int_value = 1;
-      } else if( item->type && EQUAL(item->type,"Real") ) {
+      } else if( item->type && EQUAL(item->type,"Long")) { /* signed */
+	mvt_value->sint_value = atol(value->value);
+	mvt_value->has_sint_value = 1;
+      } else if( item->type && EQUAL(item->type,"Real")) {
         mvt_value->float_value = atof(value->value);
         mvt_value->has_float_value = 1;
       } else if( item->type && EQUAL(item->type,"Boolean") ) {
