@@ -29,10 +29,10 @@
 #include "mapserver.h"
 
 MS_DLL_EXPORT char *msSLDGenerateSLD(mapObj *map, int iLayer, const char *pszVersion);
-MS_DLL_EXPORT int msSLDApplySLDURL(mapObj *map, char *szURL, int iLayer,
-                                   char *pszStyleLayerName, char **ppszLayerNames);
-MS_DLL_EXPORT int msSLDApplySLD(mapObj *map, char *psSLDXML, int iLayer,
-                                char *pszStyleLayerName, char **ppszLayerNames);
+MS_DLL_EXPORT int msSLDApplySLDURL(mapObj *map, const char *szURL, int iLayer,
+                                   const char *pszStyleLayerName, char **ppszLayerNames);
+MS_DLL_EXPORT int msSLDApplySLD(mapObj *map, const char *psSLDXML, int iLayer,
+                                const char *pszStyleLayerName, char **ppszLayerNames);
 
 #ifdef USE_OGR
 
@@ -43,7 +43,7 @@ MS_DLL_EXPORT int msSLDApplySLD(mapObj *map, char *psSLDXML, int iLayer,
 /* -------------------------------------------------------------------- */
 /*      prototypes.                                                     */
 /* -------------------------------------------------------------------- */
-layerObj  *msSLDParseSLD(mapObj *map, char *psSLDXML, int *pnLayers);
+layerObj  *msSLDParseSLD(mapObj *map, const char *psSLDXML, int *pnLayers);
 int msSLDParseNamedLayer(CPLXMLNode *psRoot, layerObj *layer);
 int msSLDParseRule(CPLXMLNode *psRoot, layerObj *psLayer);
 int msSLDParseStroke(CPLXMLNode *psStroke, styleObj *psStyle,
@@ -63,7 +63,7 @@ int msSLDParseRasterSymbolizer(CPLXMLNode *psRoot, layerObj *psLayer);
 
 int msSLDParseGraphicFillOrStroke(CPLXMLNode *psGraphicFill,
                                   char *pszDashValue,
-                                  styleObj *psStyle, mapObj *map, int bPointLayer);
+                                  styleObj *psStyle, mapObj *map);
 int msSLDParseExternalGraphic(CPLXMLNode *psExternalGraphic, styleObj *psStyle,
                               mapObj *map);
 
