@@ -760,6 +760,8 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
                 "STORAGE=%s value not supported.",
                 "msOGRWriteFromQuery()",
                 storage );
+    CSLDestroy(layer_options);
+    CSLDestroy(ds_options);
     return MS_FAILURE;
   }
 
@@ -789,6 +791,9 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
                   "Attempt to create directory '%s' failed.",
                   "msOGRWriteFromQuery()",
                   dir_to_create );
+      msFree(request_dir);
+      CSLDestroy(layer_options);
+      CSLDestroy(ds_options);
       return MS_FAILURE;
     }
   }
@@ -816,6 +821,9 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
            "Invalid value for FILENAME option. "
            "It must not contain any directory information.",
            "msOGRWriteFromQuery()" );
+    msFree(request_dir);
+    CSLDestroy(layer_options);
+    CSLDestroy(ds_options);
     return MS_FAILURE;
   }
 
@@ -876,6 +884,7 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
                 "msOGRWriteFromQuery()",
                 datasource_name,
                 format->driver+4 );
+    CSLDestroy(layer_options);
     return MS_FAILURE;
   }
 
@@ -983,6 +992,7 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
                   "msOGRWriteFromQuery()",
                   layer->name,
                   format->driver+4 );
+      CSLDestroy(layer_options);
       return MS_FAILURE;
     }
 
@@ -1051,6 +1061,7 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
         OGR_DS_Destroy( hDS );
         msOGRCleanupDS( datasource_name );
         msGMLFreeItems(item_list);
+        CSLDestroy(layer_options);
         return MS_FAILURE;
       }
 
@@ -1071,6 +1082,7 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
           OGR_DS_Destroy( hDS );
           msOGRCleanupDS( datasource_name );
           msGMLFreeItems(item_list);
+          CSLDestroy(layer_options);
           return status;
         }
       }
@@ -1101,6 +1113,7 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
             msOGRCleanupDS( datasource_name );
             msGMLFreeItems(item_list);
             msFreeShape(&resultshape);
+            CSLDestroy(layer_options);
             return status;
         }
       }
@@ -1152,6 +1165,7 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
         msOGRCleanupDS( datasource_name );
         msGMLFreeItems(item_list);
         msFreeShape(&resultshape);
+        CSLDestroy(layer_options);
         return status;
       }
     }
@@ -1165,6 +1179,8 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
   /* -------------------------------------------------------------------- */
   OGR_DS_Destroy( hDS );
 
+  CSLDestroy( layer_options );
+
   /* -------------------------------------------------------------------- */
   /*      Get list of resulting files.                                    */
   /* -------------------------------------------------------------------- */
@@ -1366,7 +1382,6 @@ int msOGRWriteFromQuery( mapObj *map, outputFormatObj *format, int sendheaders )
 
   msOGRCleanupDS( datasource_name );
 
-  CSLDestroy( layer_options );
   CSLDestroy( file_list );
 
   return MS_SUCCESS;
