@@ -37,6 +37,9 @@
 extern "C" {
 #endif
 
+/* workaround to allow compiling against Proj 6.x (#5766) */
+#define ACCEPT_USE_OF_DEPRECATED_PROJ_API_H 1
+
 #ifdef USE_PROJ
 #  include <proj_api.h>
 #if PJ_VERSION >= 470 && PJ_VERSION < 480
@@ -91,12 +94,15 @@ extern "C" {
   MS_DLL_EXPORT int msLoadProjectionString(projectionObj *p, const char *value);
   MS_DLL_EXPORT int msLoadProjectionStringEPSG(projectionObj *p, const char *value);
   MS_DLL_EXPORT char *msGetProjectionString(projectionObj *proj);
+  int msIsAxisInvertedProj( projectionObj *proj );
+  void msAxisSwapShape(shapeObj *shape);
   MS_DLL_EXPORT void msAxisNormalizePoints( projectionObj *proj, int count,
       double *x, double *y );
   MS_DLL_EXPORT void msAxisDenormalizePoints( projectionObj *proj, int count,
       double *x, double *y );
 
   MS_DLL_EXPORT void msSetPROJ_LIB( const char *, const char * );
+  MS_DLL_EXPORT void msProjLibInitFromEnv();
 
   /* Provides compatiblity with PROJ.4 4.4.2 */
 #ifndef PJ_VERSION
@@ -105,6 +111,8 @@ extern "C" {
 
   /*utility functions */
   MS_DLL_EXPORT int GetMapserverUnitUsingProj(projectionObj *psProj);
+
+  int msProjectHasLonWrap(projectionObj *in, double* pdfLonWrap);
 #endif
 
 #ifdef __cplusplus
