@@ -34,6 +34,7 @@
 /* grab mapserver declarations to wrap
  */
 #include "../../mapserver.h"
+#include "../../mapows.h"
 #include "../../maperror.h"
 #include "../../mapprimitive.h"
 #include "../../mapsymbol.h"
@@ -169,12 +170,13 @@ int mapObj_embedLegend(mapObj* self, imageObj *img)
 
 int mapObj_drawLabelCache(mapObj* self, imageObj *img)
 {
-  return msDrawLabelCache(img, self);
+  return msDrawLabelCache(self, img);
 }
 
 labelCacheMemberObj* mapObj_getLabel(mapObj* self, int i)
 {
-  return msGetLabelCacheMember(&(self->labelcache), i);
+  msSetError(MS_MISCERR, "LabelCacheMember access is not available", "getLabel()");
+  return NULL;
 }
 
 int mapObj_queryByPoint(mapObj* self, pointObj *point, int mode, double buffer)
@@ -211,13 +213,10 @@ int mapObj_queryByFilter(mapObj* self, char *string)
   msInitQuery(&(self->query));
 
   self->query.type = MS_QUERY_BY_FILTER;
+  self->query.mode = MS_QUERY_MULTIPLE;
 
-  self->query.filter = (expressionObj *) malloc(sizeof(expressionObj));
-  self->query.filter->compiled = MS_FALSE;
-  self->query.filter->flags = 0;
-  self->query.filter->tokens = self->query.filter->curtoken = NULL;
-  self->query.filter->string = strdup(string);
-  self->query.filter->type = 2000; /* MS_EXPRESSION: lot's of conflicts in mapfile.h */
+  self->query.filter.string = msStrdup(string);
+  self->query.filter.type = MS_EXPRESSION;
 
   self->query.rect = self->extent;
 
@@ -352,7 +351,7 @@ int mapObj_setSymbolSet(mapObj *self,
   msInitSymbolSet(&self->symbolset);
 
   // Set symbolset filename
-  self->symbolset.filename = strdup(szFileName);
+  self->symbolset.filename = msStrdup(szFileName);
 
   // Symbolset shares same fontset as main mapfile
   self->symbolset.fontset = &(self->fontset);
@@ -371,7 +370,7 @@ int mapObj_setFontSet(mapObj *self, char *szFileName)
   msInitFontSet(&(self->fontset));
 
   // Set fontset filename
-  self->fontset.filename = strdup(szFileName);
+  self->fontset.filename = msStrdup(szFileName);
 
   return msLoadFontSet(&(self->fontset), self);
 }
@@ -602,17 +601,22 @@ int layerObj_queryByAttributes(layerObj *self, mapObj *map, char *qitem, char *q
   int retval;
 
   msInitQuery(&(map->query));
-
-  map->query.type = MS_QUERY_BY_ATTRIBUTE;
+  
+  map->query.type = MS_QUERY_BY_FILTER;
   map->query.mode = mode;
-  if(qitem) map->query.item = strdup(qitem);
-  if(qstring) map->query.str = strdup(qstring);
+
+  if(qitem) map->query.filteritem = msStrdup(qitem);
+  if(qstring) {
+    msInitExpression(&map->query.filter);
+    msLoadExpressionString(&map->query.filter, qstring);
+  }
+
   map->query.layer = self->index;
   map->query.rect = map->extent;
 
   status = self->status;
   self->status = MS_ON;
-  retval = msQueryByAttributes(map);
+  retval = msQueryByFilter(map);
   self->status = status;
 
   return retval;
@@ -705,13 +709,10 @@ int layerObj_queryByFilter(layerObj *self, mapObj *map, char *string)
   msInitQuery(&(map->query));
 
   map->query.type = MS_QUERY_BY_FILTER;
+  map->query.mode = MS_QUERY_MULTIPLE;
 
-  map->query.filter = (expressionObj *) malloc(sizeof(expressionObj));
-  map->query.filter->string = strdup(string);
-  map->query.filter->type = 2000; /* MS_EXPRESSION: lot's of conflicts in mapfile.h */
-  map->query.filter->compiled = MS_FALSE;
-  map->query.filter->flags = 0;
-  map->query.filter->tokens = map->query.filter->curtoken = NULL;
+  map->query.filter.string = msStrdup(string);
+  map->query.filter.type = MS_EXPRESSION;
 
   map->query.layer = self->index;
   map->query.rect = map->extent;
@@ -749,7 +750,7 @@ int layerObj_queryByIndex(layerObj *self, mapObj *map, int tileindex, int shapei
 int layerObj_setFilter(layerObj *self, char *string)
 {
   if (!string || strlen(string) == 0) {
-    freeExpression(&self->filter);
+    msFreeExpression(&self->filter);
     return MS_SUCCESS;
   } else return msLoadExpressionString(&self->filter, string);
 }
@@ -927,7 +928,7 @@ int labelObj_deleteStyle(labelObj *self, int index)
 int labelObj_setExpression(labelObj *self, char *string)
 {
   if (!string || strlen(string) == 0) {
-    freeExpression(&self->expression);
+    msFreeExpression(&self->expression);
     return MS_SUCCESS;
   } else return msLoadExpressionString(&self->expression, string);
 }
@@ -940,7 +941,7 @@ char *labelObj_getExpressionString(labelObj *self)
 int labelObj_setText(labelObj *self, layerObj *layer, char *string)
 {
   if (!string || strlen(string) == 0) {
-    freeExpression(&self->text);
+    msFreeExpression(&self->text);
     return MS_SUCCESS;
   }
   return msLoadExpressionString(&self->text, string);
@@ -1035,7 +1036,6 @@ classObj *classObj_new(layerObj *layer, classObj *class)
     layer->class[layer->numclasses]->layer = layer;
   }
 
-  layer->class[layer->numclasses]->type = layer->type;
   layer->class[layer->numclasses]->layer = layer;
 
   layer->numclasses++;
@@ -1079,7 +1079,7 @@ void  classObj_destroy(classObj *self)
 int classObj_setExpression(classObj *self, char *string)
 {
   if (!string || strlen(string) == 0) {
-    freeExpression(&self->expression);
+    msFreeExpression(&self->expression);
     return MS_SUCCESS;
   } else return msLoadExpressionString(&self->expression, string);
 }
@@ -1092,7 +1092,7 @@ char *classObj_getExpressionString(classObj *self)
 int classObj_setText(classObj *self, layerObj *layer, char *string)
 {
   if (!string || strlen(string) == 0) {
-    freeExpression(&self->text);
+    msFreeExpression(&self->text);
     return MS_SUCCESS;
   }
   return msLoadExpressionString(&self->text, string);
@@ -1144,8 +1144,6 @@ classObj *classObj_clone(classObj *class, layerObj *layer)
   initClass(dstClass);
   msCopyClass(dstClass, class, layer);
 
-  dstClass->type = layer->type;
-
   return dstClass;
 }
 
@@ -1553,11 +1551,12 @@ int rectObj_draw(rectObj *self, mapObj *map, layerObj *layer,
   shape.classindex = classindex;
 
   if(text && layer->class[classindex]->numlabels > 0) {
-    shape.text = strdup(text);
-    msShapeGetAnnotation(layer,&shape);
+    shape.text = msStrdup(text);
   }
   
-  msDrawShape(map, layer, &shape, img, -1, MS_DRAWMODE_FEATURES|MS_DRAWMODE_LABELS);
+  if(MS_SUCCESS != msDrawShape(map, layer, &shape, img, -1, MS_DRAWMODE_FEATURES|MS_DRAWMODE_LABELS)) {
+    /* error message has been set already */
+  }
 
   msFreeShape(&shape);
 
@@ -1838,13 +1837,13 @@ void cgirequestObj_setParameter(cgiRequestObj *self, char *name, char *value)
   for (i=0; i<self->NumParams; i++) {
     if (strcasecmp(self->ParamNames[i], name) == 0) {
       free(self->ParamValues[i]);
-      self->ParamValues[i] = strdup(value);
+      self->ParamValues[i] = msStrdup(value);
       break;
     }
   }
   if (i == self->NumParams) {
-    self->ParamNames[self->NumParams] = strdup(name);
-    self->ParamValues[self->NumParams] = strdup(value);
+    self->ParamNames[self->NumParams] = msStrdup(name);
+    self->ParamValues[self->NumParams] = msStrdup(value);
     self->NumParams++;
   }
 }
@@ -1854,8 +1853,8 @@ void cgirequestObj_addParameter(cgiRequestObj *self, char *name, char *value)
   if (self->NumParams == MS_DEFAULT_CGI_PARAMS) {
     msSetError(MS_CHILDERR, "Maximum number of items, %d, has been reached", "addParameter()", MS_DEFAULT_CGI_PARAMS);
   }
-  self->ParamNames[self->NumParams] = strdup(name);
-  self->ParamValues[self->NumParams] = strdup(value);
+  self->ParamNames[self->NumParams] = msStrdup(name);
+  self->ParamValues[self->NumParams] = msStrdup(value);
   self->NumParams++;
 }
 
@@ -1963,7 +1962,7 @@ char *clusterObj_convertToString(clusterObj *self)
 int clusterObj_setGroup(clusterObj *self, char *string)
 {
   if (!string || strlen(string) == 0) {
-    freeExpression(&self->group);
+    msFreeExpression(&self->group);
     return MS_SUCCESS;
   } else return msLoadExpressionString(&self->group, string);
 }
@@ -1976,7 +1975,7 @@ char *clusterObj_getGroupString(clusterObj *self)
 int clusterObj_setFilter(clusterObj *self, char *string)
 {
   if (!string || strlen(string) == 0) {
-    freeExpression(&self->filter);
+    msFreeExpression(&self->filter);
     return MS_SUCCESS;
   } else return msLoadExpressionString(&self->filter, string);
 }
@@ -2020,6 +2019,7 @@ imageObj *symbolObj_getImage(symbolObj *self, outputFormatObj *input_format)
   imageObj *image = NULL;
   outputFormatObj *format = NULL;
   rendererVTableObj *renderer = NULL;
+  int retval;
 
   if (self->type != MS_SYMBOL_PIXMAP) {
     msSetError(MS_SYMERR, "Can't return image from non-pixmap symbol",
@@ -2030,9 +2030,7 @@ imageObj *symbolObj_getImage(symbolObj *self, outputFormatObj *input_format)
   if (input_format) {
     format = input_format;
   } else {
-    format = msCreateDefaultOutputFormat(NULL, "GD/GIF", "gdgif");
-    if (format == NULL)
-      format = msCreateDefaultOutputFormat(NULL, "GD/PNG", "gdpng");
+    format = msCreateDefaultOutputFormat(NULL, "AGG/PNG", "png");
 
     if (format)
       msInitializeRendererVTable(format);
@@ -2049,8 +2047,15 @@ imageObj *symbolObj_getImage(symbolObj *self, outputFormatObj *input_format)
   if (self->pixmap_buffer) {
     image = msImageCreate(self->pixmap_buffer->width, self->pixmap_buffer->height, format, NULL, NULL,
                           MS_DEFAULT_RESOLUTION, MS_DEFAULT_RESOLUTION, NULL);
-    renderer->mergeRasterBuffer(image, self->pixmap_buffer, 1.0, 0, 0, 0, 0,
+    if(!image) {
+      return NULL;
+    }
+    retval = renderer->mergeRasterBuffer(image, self->pixmap_buffer, 1.0, 0, 0, 0, 0,
                                 self->pixmap_buffer->width, self->pixmap_buffer->height);
+    if(retval != MS_SUCCESS) {
+      msFreeImage(image);
+      return NULL;
+    }
   }
 
   return image;
@@ -2073,7 +2078,8 @@ int symbolObj_setImage(symbolObj *self, imageObj *image)
     return MS_FAILURE;
   }
   self->type = MS_SYMBOL_PIXMAP;
-  renderer->getRasterBufferCopy(image, self->pixmap_buffer);
+  if(renderer->getRasterBufferCopy(image, self->pixmap_buffer) != MS_SUCCESS)
+    return MS_FAILURE;
 
   return MS_SUCCESS;
 }
