@@ -527,7 +527,7 @@ error:
   return;
 }
 
-ms_bitarray msSearchDiskTree(const char *filename, rectObj aoi, int debug)
+ms_bitarray msSearchDiskTree(const char *filename, rectObj aoi, int debug, int numshapes)
 {
   SHPTreeHandle disktree;
   ms_bitarray status=NULL;
@@ -541,6 +541,12 @@ ms_bitarray msSearchDiskTree(const char *filename, rectObj aoi, int debug)
     return(NULL);
   }
 
+  if (disktree->nShapes != numshapes) {
+      msSetError(MS_SHPERR, "The spatial index file %s is corrupt.", "msSearchDiskTree()", filename);
+      msSHPDiskTreeClose(disktree);
+      return(NULL);
+  }
+
   status = msAllocBitArray(disktree->nShapes);
   if(!status) {
     msSetError(MS_MEMERR, NULL, "msSearchDiskTree()");
